/***********************************************************
Copyright 1997-2002,2008 by Tadayoshi Funaba, Tokyo, Japan. tadf@funaba.org

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Tadayoshi Funaba
not be used in advertising or publicity pertaining to distribution
of the software without specific, written prior permission.

TADAYOSHI FUNABA DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL TADAYOSHI FUNABA BE LIABLE FOR ANY SPECIAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

$Id: calendar2module.c,v 1.12 2008-11-12 19:07:37+09 tadf Exp $

******************************************************************/

#include <Python.h>
#include "calendar.h"
#include "calendar2.h"
#include "qref.h"

#define CHECK_RANGE(v, m)			\
{						\
    if ((v) < (m)) {				\
	PyErr_SetString				\
	    (PyExc_ValueError,			\
	     "out of range");			\
	return NULL;				\
    }						\
}

#define CHECK_DOMAIN(v, m)			\
{						\
    if ((v) < (m)) {				\
	PyErr_SetString				\
	    (PyExc_ValueError,			\
	     "out of domain");			\
	return NULL;				\
    }						\
}

#define CHECK_DOMAIN2(v, m, n)			\
{						\
    if ((v) < (m) || (v) > (n)) {		\
	PyErr_SetString				\
	    (PyExc_ValueError,			\
	     "out of domain");			\
	return NULL;				\
    }						\
}

static PyObject *
calendar2_gregorian_leap_year(PyObject *self, PyObject *args)
{
    int y, l;

    if (!PyArg_ParseTuple(args, "i", &y))
	return NULL;
    CHECK_DOMAIN(y, 1);
    l = gregorian_leap_year(y);
    return Py_BuildValue("i", l);
}

static PyObject *
calendar2_gregorian_last_day_of_month(PyObject *self, PyObject *args)
{
    int m, y, md;

    if (!PyArg_ParseTuple(args, "ii", &m, &y))
	return NULL;
    CHECK_DOMAIN2(m, 1, 12);
    CHECK_DOMAIN(y, 1);
    md = gregorian_last_day_of_month(m, y);
    return Py_BuildValue("i", md);
}

static PyObject *
calendar2_absolute_from_gregorian(PyObject *self, PyObject *args)
{
    int m, md, y, a, m2, md2, y2;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(iii)", &m, &md, &y))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iii", &m, &md, &y))
	    return NULL;
    }
    CHECK_DOMAIN2(m, 1, 12);
    CHECK_DOMAIN2(md, 1, 31);
    CHECK_DOMAIN(y, 1);
    a = absolute_from_gregorian(m, md, y);
    CHECK_RANGE(a, 1);
    gregorian_from_absolute(a, &m2, &md2, &y2);
    if (m != m2 || md != md2 || y != y2) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("i", a);
}

static PyObject *
calendar2_gregorian_from_absolute(PyObject *self, PyObject *args)
{
    int a, m, md, y;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 1);
    gregorian_from_absolute(a, &m, &md, &y);
    return Py_BuildValue("(iii)", m, md, y);
}

static PyObject *
calendar2_Kday_on_or_before(PyObject *self, PyObject *args)
{
    int a, k, b;

    if (!PyArg_ParseTuple(args, "ii", &a, &k))
	return NULL;
    CHECK_DOMAIN(a, 1);
    CHECK_DOMAIN2(k, 0, 6);
    b = Kday_on_or_before(a, k);
    CHECK_RANGE(b, 1);
    return Py_BuildValue("i", b);
}

static PyObject *
calendar2_absolute_from_iso(PyObject *self, PyObject *args)
{
    int w, wd, y, a, w2, wd2, y2;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(iii)", &w, &wd, &y))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iii", &w, &wd, &y))
	    return NULL;
    }
    CHECK_DOMAIN2(w, 1, 53);
    CHECK_DOMAIN2(wd, 1, 7);
    CHECK_DOMAIN(y, 1);
    a = absolute_from_iso(w, wd, y);
    CHECK_RANGE(a, 1);
    iso_from_absolute(a, &w2, &wd2, &y2);
    if (w != w2 || wd != wd2 || y != y2) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("i", a);
}

static PyObject *
calendar2_iso_from_absolute(PyObject *self, PyObject *args)
{
    int a, w, wd, y;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 1);
    iso_from_absolute(a, &w, &wd, &y);
    return Py_BuildValue("(iii)", w, wd, y);
}

static PyObject *
calendar2_julian_leap_year(PyObject *self, PyObject *args)
{
    int y, l;

    if (!PyArg_ParseTuple(args, "i", &y))
	return NULL;
    CHECK_DOMAIN(y, 1);
    l = julian_leap_year(y);
    return Py_BuildValue("i", l);
}

static PyObject *
calendar2_julian_last_day_of_month(PyObject *self, PyObject *args)
{
    int m, y, md;

    if (!PyArg_ParseTuple(args, "ii", &m, &y))
	return NULL;
    CHECK_DOMAIN2(m, 1, 12);
    CHECK_DOMAIN(y, 1);
    md = julian_last_day_of_month(m, y);
    return Py_BuildValue("i", md);
}

static PyObject *
calendar2_absolute_from_julian(PyObject *self, PyObject *args)
{
    int m, md, y, a, m2, md2, y2;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(iii)", &m, &md, &y))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iii", &m, &md, &y))
	    return NULL;
    }
    CHECK_DOMAIN2(m, 1, 12);
    CHECK_DOMAIN2(md, 1, 31);
    CHECK_DOMAIN(y, 1);
    a = absolute_from_julian(m, md, y);
    CHECK_RANGE(a, 1);
    julian_from_absolute(a, &m2, &md2, &y2);
    if (m != m2 || md != md2 || y != y2) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("i", a);
}

static PyObject *
calendar2_julian_from_absolute(PyObject *self, PyObject *args)
{
    int a, m, md, y;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 1);
    julian_from_absolute(a, &m, &md, &y);
    return Py_BuildValue("(iii)", m, md, y);
}

static PyObject *
calendar2_islamic_leap_year(PyObject *self, PyObject *args)
{
    int y, l;

    if (!PyArg_ParseTuple(args, "i", &y))
	return NULL;
    CHECK_DOMAIN(y, 1);
    l = islamic_leap_year(y);
    return Py_BuildValue("i", l);
}

static PyObject *
calendar2_islamic_last_day_of_month(PyObject *self, PyObject *args)
{
    int m, y, md;

    if (!PyArg_ParseTuple(args, "ii", &m, &y))
	return NULL;
    CHECK_DOMAIN2(m, 1, 12);
    CHECK_DOMAIN(y, 1);
    md = islamic_last_day_of_month(m, y);
    return Py_BuildValue("i", md);
}

static PyObject *
calendar2_absolute_from_islamic(PyObject *self, PyObject *args)
{
    int m, md, y, a, m2, md2, y2;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(iii)", &m, &md, &y))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iii", &m, &md, &y))
	    return NULL;
    }
    CHECK_DOMAIN2(m, 1, 12);
    CHECK_DOMAIN2(md, 1, 30);
    CHECK_DOMAIN(y, 1);
    a = absolute_from_islamic(m, md, y);
    CHECK_RANGE(a, 1); /* 227015 */
    islamic_from_absolute(a, &m2, &md2, &y2);
    if (m != m2 || md != md2 || y != y2) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("i", a);
}

static PyObject *
calendar2_islamic_from_absolute(PyObject *self, PyObject *args)
{
    int a, m, md, y;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 227015);
    islamic_from_absolute(a, &m, &md, &y);
    return Py_BuildValue("(iii)", m, md, y);
}

static PyObject *
calendar2_hebrew_leap_year(PyObject *self, PyObject *args)
{
    int y, l;

    if (!PyArg_ParseTuple(args, "i", &y))
	return NULL;
    CHECK_DOMAIN(y, 3761);
    l = hebrew_leap_year(y);
    return Py_BuildValue("i", l);
}

static PyObject *
calendar2_last_month_of_hebrew_year(PyObject *self, PyObject *args)
{
    int y, m;

    if (!PyArg_ParseTuple(args, "i", &y))
	return NULL;
    CHECK_DOMAIN(y, 1);
    m = last_month_of_hebrew_year(y);
    return Py_BuildValue("i", m);
}

static PyObject *
calendar2_hebrew_last_day_of_month(PyObject *self, PyObject *args)
{
    int m, y, md;

    if (!PyArg_ParseTuple(args, "ii", &m, &y))
	return NULL;
    CHECK_DOMAIN2(m, 1, 13);
    CHECK_DOMAIN(y, 1);
    md = hebrew_last_day_of_month(m, y);
    return Py_BuildValue("i", md);
}

static PyObject *
calendar2_hebrew_calendar_elapsed_days(PyObject *self, PyObject *args)
{
    int y, d;

    if (!PyArg_ParseTuple(args, "i", &y))
	return NULL;
    CHECK_DOMAIN(y, 1);
    d = hebrew_calendar_elapsed_days(y);
    return Py_BuildValue("i", d);
}

static PyObject *
calendar2_days_in_hebrew_year(PyObject *self, PyObject *args)
{
    int y, d;

    if (!PyArg_ParseTuple(args, "i", &y))
	return NULL;
    CHECK_DOMAIN(y, 1);
    d = days_in_hebrew_year(y);
    return Py_BuildValue("i", d);
}

static PyObject *
calendar2_long_heshvan(PyObject *self, PyObject *args)
{
    int y, l;

    if (!PyArg_ParseTuple(args, "i", &y))
	return NULL;
    CHECK_DOMAIN(y, 1);
    l = long_heshvan(y);
    return Py_BuildValue("i", l);
}

static PyObject *
calendar2_short_kislev(PyObject *self, PyObject *args)
{
    int y, s;

    if (!PyArg_ParseTuple(args, "i", &y))
	return NULL;
    CHECK_DOMAIN(y, 1);
    s = short_kislev(y);
    return Py_BuildValue("i", s);
}

static PyObject *
calendar2_absolute_from_hebrew(PyObject *self, PyObject *args)
{
    int m, md, y, a, m2, md2, y2;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(iii)", &m, &md, &y))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iii", &m, &md, &y))
	    return NULL;
    }
    CHECK_DOMAIN2(m, 1, 13);
    CHECK_DOMAIN2(md, 1, 30);
    CHECK_DOMAIN(y, 1);
    a = absolute_from_hebrew(m, md, y);
    CHECK_RANGE(a, 1);
    hebrew_from_absolute(a, &m2, &md2, &y2);
    if (m != m2 || md != md2 || y != y2) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("i", a);
}

static PyObject *
calendar2_hebrew_from_absolute(PyObject *self, PyObject *args)
{
    int a, m, md, y;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 1);
    hebrew_from_absolute(a, &m, &md, &y);
    return Py_BuildValue("(iii)", m, md, y);
}

static PyObject *
calendar2_Nth_Kday(PyObject *self, PyObject *args)
{
    int n, k, m, y, b;

    if (!PyArg_ParseTuple(args, "iiii", &n, &k, &m, &y))
	return NULL;
    if (n < -5 || n == 0 || n > 5)
	PyErr_SetString(PyExc_ValueError, "out of range");
    CHECK_DOMAIN2(k, 0, 6);
    CHECK_DOMAIN2(m, 1, 12);
    CHECK_DOMAIN(y, 1);
    b = Nth_Kday(n, k, m, y);
    CHECK_RANGE(b, 1);
    return Py_BuildValue("i", b);
}

static PyObject *
calendar2_nicaean_rule_easter(PyObject *self, PyObject *args)
{
    int y, a;

    if (!PyArg_ParseTuple(args, "i", &y))
	return NULL;
    CHECK_DOMAIN(y, 1);
    a = nicaean_rule_easter(y);
    return Py_BuildValue("i", a);
}

static PyObject *
calendar2_easter(PyObject *self, PyObject *args)
{
    int y, a;

    if (!PyArg_ParseTuple(args, "i", &y))
	return NULL;
    CHECK_DOMAIN(y, 1);
    a = easter(y);
    return Py_BuildValue("i", a);
}

static PyObject *
calendar2_absolute_from_mayan_long_count(PyObject *self, PyObject *args)
{
    int bt, kt, t, u, k, a, bt2, kt2, t2, u2, k2;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(iiiii)", &bt, &kt, &t, &u, &k))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iiiii", &bt, &kt, &t, &u, &k))
	    return NULL;
    }
    CHECK_DOMAIN2(bt, 0, 19);
    CHECK_DOMAIN2(kt, 0, 19);
    CHECK_DOMAIN2(t, 0, 19);
    CHECK_DOMAIN2(u, 0, 17);
    CHECK_DOMAIN2(k, 0, 19);
    a = absolute_from_mayan_long_count(bt, kt, t, u, k);
    CHECK_RANGE(a, 1);
    mayan_long_count_from_absolute(a, &bt2, &kt2, &t2, &u2, &k2);
    if (bt != bt2 || kt != kt2 || t != t2 || u != u2 || k != k2) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("i", a);
}

static PyObject *
calendar2_mayan_long_count_from_absolute(PyObject *self, PyObject *args)
{
    int a, bt, kt, t, u, k;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 1);
    mayan_long_count_from_absolute(a, &bt, &kt, &t, &u, &k);
    return Py_BuildValue("(iiiii)", bt, kt, t, u, k);
}

static PyObject *
calendar2_mayan_haab_from_absolute(PyObject *self, PyObject *args)
{
    int a, md, m;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 1);
    mayan_haab_from_absolute(a, &md, &m);
    return Py_BuildValue("(ii)", md, m);
}

static PyObject *
calendar2_mayan_haab_difference(PyObject *self, PyObject *args)
{
    int md1, m1, md2, m2, d;

    if (PyTuple_Size(args) == 2) {
	if (!PyArg_ParseTuple(args, "(ii)(ii)", &md1, &m1, &md2, &m2))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iiii", &md1, &m1, &md2, &m2))
	    return NULL;
    }
    CHECK_DOMAIN2(md1, 0, 19);
    CHECK_DOMAIN2(m1, 1, 19);
    CHECK_DOMAIN2(md2, 0, 19);
    CHECK_DOMAIN2(m2, 1, 19);
    d = mayan_haab_difference(md1, m1, md2, m2);
    return Py_BuildValue("i", d);
}

static PyObject *
calendar2_mayan_haab_on_or_before(PyObject *self, PyObject *args)
{
    int md, m, a, d;

    if (PyTuple_Size(args) == 2) {
	if (!PyArg_ParseTuple(args, "(ii)i", &md, &m, &a))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iii", &md, &m, &a))
	    return NULL;
    }
    CHECK_DOMAIN2(md, 0, 19);
    CHECK_DOMAIN2(m, 1, 19);
    CHECK_DOMAIN(a, 1);
    d = mayan_haab_on_or_before(md, m, a);
    CHECK_RANGE(d, 1);
    return Py_BuildValue("i", d);
}

static PyObject *
calendar2_mayan_tzolkin_from_absolute(PyObject *self, PyObject *args)
{
    int a, nu, na;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 1);
    mayan_tzolkin_from_absolute(a, &nu, &na);
    return Py_BuildValue("(ii)", nu, na);
}

static PyObject *
calendar2_mayan_tzolkin_difference(PyObject *self, PyObject *args)
{
    int nu1, na1, nu2, na2, d;

    if (PyTuple_Size(args) == 2) {
	if (!PyArg_ParseTuple(args, "(ii)(ii)", &nu1, &na1, &nu2, &na2))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iiii", &nu1, &na1, &nu2, &na2))
	    return NULL;
    }
    CHECK_DOMAIN2(nu1, 1, 13);
    CHECK_DOMAIN2(na1, 1, 20);
    CHECK_DOMAIN2(nu2, 1, 13);
    CHECK_DOMAIN2(na2, 1, 20);
    d = mayan_tzolkin_difference(nu1, na1, nu2, na2);
    return Py_BuildValue("i", d);
}

static PyObject *
calendar2_mayan_tzolkin_on_or_before(PyObject *self, PyObject *args)
{
    int nu, na, a, d;

    if (PyTuple_Size(args) == 2) {
	if (!PyArg_ParseTuple(args, "(ii)i", &nu, &na, &a))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iii", &nu, &na, &a))
	    return NULL;
    }
    CHECK_DOMAIN2(nu, 1, 13);
    CHECK_DOMAIN2(na, 1, 20);
    CHECK_DOMAIN(a, 1);
    d = mayan_tzolkin_on_or_before(nu, na, a);
    CHECK_RANGE(d, 1);
    return Py_BuildValue("i", d);
}

static PyObject *
calendar2_mayan_haab_tzolkin_on_or_before(PyObject *self, PyObject *args)
{
    int md, m, nu, na, a, d;

    if (PyTuple_Size(args) == 3) {
	if (!PyArg_ParseTuple(args, "(ii)(ii)i", &md, &m, &nu, &na, &a))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iiiii", &md, &m, &nu, &na, &a))
	    return NULL;
    }
    CHECK_DOMAIN2(md, 0, 19);
    CHECK_DOMAIN2(m, 1, 19);
    CHECK_DOMAIN2(nu, 1, 13);
    CHECK_DOMAIN2(na, 1, 20);
    CHECK_DOMAIN(a, 1);
    d = mayan_haab_tzolkin_on_or_before(md, m, nu, na, a);
    CHECK_RANGE(d, 1);
    return Py_BuildValue("i", d);
}

static PyObject *
calendar2_french_last_day_of_month(PyObject *self, PyObject *args)
{
    int m, y, md;

    if (!PyArg_ParseTuple(args, "ii", &m, &y))
	return NULL;
    CHECK_DOMAIN2(m, 1, 13);
    CHECK_DOMAIN(y, 1);
    md = french_last_day_of_month(m, y);
    return Py_BuildValue("i", md);
}

static PyObject *
calendar2_french_leap_year(PyObject *self, PyObject *args)
{
    int y, l;

    if (!PyArg_ParseTuple(args, "i", &y))
	return NULL;
    CHECK_DOMAIN(y, 1);
    l = french_leap_year(y);
    return Py_BuildValue("i", l);
}

static PyObject *
calendar2_absolute_from_french(PyObject *self, PyObject *args)
{
    int m, md, y, a, m2, md2, y2;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(iii)", &m, &md, &y))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iii", &m, &md, &y))
	    return NULL;
    }
    CHECK_DOMAIN2(m, 1, 13);
    CHECK_DOMAIN2(md, 1, 30);
    CHECK_DOMAIN(y, 1);
    a = absolute_from_french(m, md, y);
    CHECK_RANGE(a, 1); /* 654415 */
    french_from_absolute(a, &m2, &md2, &y2);
    if (m != m2 || md != md2 || y != y2) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("i", a);
}

static PyObject *
calendar2_french_from_absolute(PyObject *self, PyObject *args)
{
    int a, m, md, y;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 654415);
    french_from_absolute(a, &m, &md, &y);
    return Py_BuildValue("(iii)", m, md, y);
}

static PyObject *
calendar2_solar_longitude(PyObject *self, PyObject *args)
{
    double d, l;

    if (!PyArg_ParseTuple(args, "d", &d))
	return NULL;
    l = solar_longitude(d);
    return Py_BuildValue("d", l);
}

static PyObject *
calendar2_zodiac(PyObject *self, PyObject *args)
{
    double d, l;

    if (!PyArg_ParseTuple(args, "d", &d))
	return NULL;
    l = zodiac(d);
    return Py_BuildValue("d", l);
}

static PyObject *
calendar2_old_hindu_solar_from_absolute(PyObject *self, PyObject *args)
{
    int a, m, md, y;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 1);
    old_hindu_solar_from_absolute(a, &m, &md, &y);
    return Py_BuildValue("(iii)", m, md, y);
}

static PyObject *
calendar2_absolute_from_old_hindu_solar(PyObject *self, PyObject *args)
{
    int m, md, y, a, m2, md2, y2;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(iii)", &m, &md, &y))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iii", &m, &md, &y))
	    return NULL;
    }
    CHECK_DOMAIN2(m, 1, 12);
    CHECK_DOMAIN2(md, 1, 31);
    CHECK_DOMAIN(y, 1);
    a = absolute_from_old_hindu_solar(m, md, y);
    CHECK_RANGE(a, 1);
    old_hindu_solar_from_absolute(a, &m2, &md2, &y2);
    if (m != m2 || md != md2 || y != y2) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("i", a);
}

static PyObject *
calendar2_lunar_longitude(PyObject *self, PyObject *args)
{
    double d, l;

    if (!PyArg_ParseTuple(args, "d", &d))
	return NULL;
    l = lunar_longitude(d);
    return Py_BuildValue("d", l);
}

static PyObject *
calendar2_lunar_phase(PyObject *self, PyObject *args)
{
    double d, p;

    if (!PyArg_ParseTuple(args, "d", &d))
	return NULL;
    p = lunar_phase(d);
    return Py_BuildValue("d", p);
}

static PyObject *
calendar2_new_moon(PyObject *self, PyObject *args)
{
    double d, n;

    if (!PyArg_ParseTuple(args, "d", &d))
	return NULL;
    n = new_moon(d);
    return Py_BuildValue("d", n);
}

static PyObject *
calendar2_old_hindu_lunar_from_absolute(PyObject *self, PyObject *args)
{
    int a, m, l, md, y;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 1);
    old_hindu_lunar_from_absolute(a, &m, &l, &md, &y);
    return Py_BuildValue("(iiii)", m, l, md, y);
}

static PyObject *
calendar2_old_hindu_lunar_precedes(PyObject *self, PyObject *args)
{
    int m1, l1, md1, y1, m2, l2, md2, y2, d;

    if (PyTuple_Size(args) == 2) {
	if (!PyArg_ParseTuple
	    (args, "(iiii)(iiii)", &m1, &l1, &md1, &y1, &m2, &l2, &md2, &y2))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple
	    (args, "iiiiiiii", &m1, &l1, &md1, &y1, &m2, &l2, &md2, &y2))
	    return NULL;
    }
    CHECK_DOMAIN2(m1, 1, 12);
    CHECK_DOMAIN2(md1, 1, 30);
    CHECK_DOMAIN(y1, 1);
    CHECK_DOMAIN2(m2, 1, 12);
    CHECK_DOMAIN2(md2, 1, 30);
    CHECK_DOMAIN(y2, 1);
    d = old_hindu_lunar_precedes(m1, l1, md1, y1, m2, l2, md2, y2);
    return Py_BuildValue("i", d);
}

static PyObject *
calendar2_absolute_from_old_hindu_lunar(PyObject *self, PyObject *args)
{
    int m, l, md, y, a, m2, l2, md2, y2;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(iiii)", &m, &l, &md, &y))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iiii", &m, &l, &md, &y))
	    return NULL;
    }
    CHECK_DOMAIN2(m, 1, 12);
    CHECK_DOMAIN2(md, 1, 30);
    CHECK_DOMAIN(y, 1);
    a = absolute_from_old_hindu_lunar(m, l, md, y);
    CHECK_RANGE(a, 1);
    old_hindu_lunar_from_absolute(a, &m2, &l2, &md2, &y2);
    if (m != m2 || l != l2 || md != md2 || y != y2) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("i", a);
}

static PyObject *
calendar2_world_leap_year(PyObject *self, PyObject *args)
{
    int y, l;

    if (!PyArg_ParseTuple(args, "i", &y))
	return NULL;
    CHECK_DOMAIN(y, 1);
    l = world_leap_year(y);
    return Py_BuildValue("i", l);
}

static PyObject *
calendar2_world_last_day_of_month(PyObject *self, PyObject *args)
{
    int m, y, md;

    if (!PyArg_ParseTuple(args, "ii", &m, &y))
	return NULL;
    CHECK_DOMAIN2(m, 1, 12);
    CHECK_DOMAIN(y, 1);
    md = world_last_day_of_month(m, y);
    return Py_BuildValue("i", md);
}

static PyObject *
calendar2_absolute_from_world(PyObject *self, PyObject *args)
{
    int m, md, y, a, m2, md2, y2;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(iii)", &m, &md, &y))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iii", &m, &md, &y))
	    return NULL;
    }
    CHECK_DOMAIN2(m, 1, 12);
    CHECK_DOMAIN2(md, 1, 31);
    CHECK_DOMAIN(y, 1);
    a = absolute_from_world(m, md, y);
    CHECK_RANGE(a, 1);
    world_from_absolute(a, &m2, &md2, &y2);
    if (m != m2 || md != md2 || y != y2) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("i", a);
}

static PyObject *
calendar2_world_from_absolute(PyObject *self, PyObject *args)
{
    int a, m, md, y;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 1);
    world_from_absolute(a, &m, &md, &y);
    return Py_BuildValue("(iii)", m, md, y);
}

static PyObject *
calendar2_world_day_of_week_from_absolute(PyObject *self, PyObject *args)
{
    int a, w;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 1);
    w = world_day_of_week_from_absolute(a);
    return Py_BuildValue("i", w);
}

static PyObject *
calendar2_coptic_leap_year(PyObject *self, PyObject *args)
{
    int y, l;

    if (!PyArg_ParseTuple(args, "i", &y))
	return NULL;
    CHECK_DOMAIN(y, 1);
    l = coptic_leap_year(y);
    return Py_BuildValue("i", l);
}

static PyObject *
calendar2_coptic_last_day_of_month(PyObject *self, PyObject *args)
{
    int m, y, md;

    if (!PyArg_ParseTuple(args, "ii", &m, &y))
	return NULL;
    CHECK_DOMAIN2(m, 1, 13);
    CHECK_DOMAIN(y, 1);
    md = coptic_last_day_of_month(m, y);
    return Py_BuildValue("i", md);
}

static PyObject *
calendar2_absolute_from_coptic(PyObject *self, PyObject *args)
{
    int m, md, y, a, m2, md2, y2;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(iii)", &m, &md, &y))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iii", &m, &md, &y))
	    return NULL;
    }
    CHECK_DOMAIN2(m, 1, 13);
    CHECK_DOMAIN2(md, 1, 30);
    CHECK_DOMAIN(y, 1);
    a = absolute_from_coptic(m, md, y);
    CHECK_RANGE(a, 1); /* 103605 */
    coptic_from_absolute(a, &m2, &md2, &y2);
    if (m != m2 || md != md2 || y != y2) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("i", a);
}

static PyObject *
calendar2_coptic_from_absolute(PyObject *self, PyObject *args)
{
    int a, m, md, y;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 103605);
    coptic_from_absolute(a, &m, &md, &y);
    return Py_BuildValue("(iii)", m, md, y);
}

static PyObject *
calendar2_ethiopian_leap_year(PyObject *self, PyObject *args)
{
    int y, l;

    if (!PyArg_ParseTuple(args, "i", &y))
	return NULL;
    CHECK_DOMAIN(y, 1);
    l = ethiopian_leap_year(y);
    return Py_BuildValue("i", l);
}

static PyObject *
calendar2_ethiopian_last_day_of_month(PyObject *self, PyObject *args)
{
    int m, y, md;

    if (!PyArg_ParseTuple(args, "ii", &m, &y))
	return NULL;
    CHECK_DOMAIN2(m, 1, 13);
    CHECK_DOMAIN(y, 1);
    md = ethiopian_last_day_of_month(m, y);
    return Py_BuildValue("i", md);
}

static PyObject *
calendar2_absolute_from_ethiopian(PyObject *self, PyObject *args)
{
    int m, md, y, a, m2, md2, y2;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(iii)", &m, &md, &y))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iii", &m, &md, &y))
	    return NULL;
    }
    CHECK_DOMAIN2(m, 1, 13);
    CHECK_DOMAIN2(md, 1, 30);
    CHECK_DOMAIN(y, 1);
    a = absolute_from_ethiopian(m, md, y);
    CHECK_RANGE(a, 1); /* 2796 */
    ethiopian_from_absolute(a, &m2, &md2, &y2);
    if (m != m2 || md != md2 || y != y2) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("i", a);
}

static PyObject *
calendar2_ethiopian_from_absolute(PyObject *self, PyObject *args)
{
    int a, m, md, y;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 2796);
    ethiopian_from_absolute(a, &m, &md, &y);
    return Py_BuildValue("(iii)", m, md, y);
}

static PyObject *
calendar2_jalaali_leap_year(PyObject *self, PyObject *args)
{
    int y, l;

    if (!PyArg_ParseTuple(args, "i", &y))
	return NULL;
    CHECK_DOMAIN(y, 1);
    l = jalaali_leap_year(y);
    return Py_BuildValue("i", l);
}

static PyObject *
calendar2_jalaali_last_day_of_month(PyObject *self, PyObject *args)
{
    int m, y, md;

    if (!PyArg_ParseTuple(args, "ii", &m, &y))
	return NULL;
    CHECK_DOMAIN2(m, 1, 12);
    CHECK_DOMAIN(y, 1);
    md = jalaali_last_day_of_month(m, y);
    return Py_BuildValue("i", md);
}

static PyObject *
calendar2_absolute_from_jalaali(PyObject *self, PyObject *args)
{
    int m, md, y, a, m2, md2, y2;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(iii)", &m, &md, &y))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iii", &m, &md, &y))
	    return NULL;
    }
    CHECK_DOMAIN2(m, 1, 12);
    CHECK_DOMAIN2(md, 1, 31);
    CHECK_DOMAIN(y, 1);
    a = absolute_from_jalaali(m, md, y);
    CHECK_RANGE(a, 1);  /* 632680 - 823335 */
    jalaali_from_absolute(a, &m2, &md2, &y2);
    if (m != m2 || md != md2 || y != y2) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("i", a);
}

static PyObject *
calendar2_absolute_from_ordinal_calendar(PyObject *self, PyObject *args)
{
    int yd, y, a, yd2, y2;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(ii)", &yd, &y))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "ii", &yd, &y))
	    return NULL;
    }
    CHECK_DOMAIN2(yd, 1, 366);
    CHECK_DOMAIN(y, 1);
    a = absolute_from_ordinal_calendar(yd, y);
    CHECK_RANGE(a, 1);
    ordinal_calendar_from_absolute(a, &yd2, &y2);
    if (yd != yd2 || y != y2) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("i", a);

}

static PyObject *
calendar2_ordinal_calendar_from_absolute(PyObject *self, PyObject *args)
{
    int a, yd, y;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 1);
    ordinal_calendar_from_absolute(a, &yd, &y);
    return Py_BuildValue("(ii)", yd, y);
}

static PyObject *
calendar2_absolute_from_julian_day_number(PyObject *self, PyObject *args)
{
    int j, a;

    if (!PyArg_ParseTuple(args, "i", &j))
	return NULL;
    CHECK_DOMAIN(j, 1721426);
    a = absolute_from_julian_day_number(j);
    CHECK_RANGE(a, 1);
    return Py_BuildValue("i", a);
}

static PyObject *
calendar2_julian_day_number_from_absolute(PyObject *self, PyObject *args)
{
    int a, j;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 1);
    j = julian_day_number_from_absolute(a);
    CHECK_RANGE(j, 1721426);
    return Py_BuildValue("i", j);
}

static PyObject *
calendar2_julian_day_number_from_mjd(PyObject *self, PyObject *args)
{
    int m, j;

    if (!PyArg_ParseTuple(args, "i", &m))
	return NULL;
    CHECK_DOMAIN(m, -678576);
    j = julian_day_number_from_mjd(m);
    CHECK_RANGE(j, 1721426);
    return Py_BuildValue("i", j);
}

static PyObject *
calendar2_mjd_from_julian_day_number(PyObject *self, PyObject *args)
{
    int j, m;

    if (!PyArg_ParseTuple(args, "i", &j))
	return NULL;
    CHECK_DOMAIN(j, 1721426);
    m = mjd_from_julian_day_number(j);
    CHECK_RANGE(m, -678576);
    return Py_BuildValue("i", m);
}

static PyObject *
calendar2_absolute_from_lilian_day_number(PyObject *self, PyObject *args)
{
    int l, a;

    if (!PyArg_ParseTuple(args, "i", &l))
	return NULL;
    CHECK_DOMAIN(l, -577734);
    a = absolute_from_lilian_day_number(l);
    CHECK_RANGE(a, 1);
    return Py_BuildValue("i", a);
}

static PyObject *
calendar2_lilian_day_number_from_absolute(PyObject *self, PyObject *args)
{
    int a, l;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 1);
    l = lilian_day_number_from_absolute(a);
    CHECK_RANGE(l, -577734);
    return Py_BuildValue("i", l);
}

static PyObject *
calendar2_day_of_week_from_absolute(PyObject *self, PyObject *args)
{
    int a, w;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN(a, 1);
    w = day_of_week_from_absolute(a);
    return Py_BuildValue("i", w);
}

static PyObject *
calendar2_jalaali_from_absolute(PyObject *self, PyObject *args)
{
    int a, m, md, y;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN2(a, 632680, 823335);
    jalaali_from_absolute(a, &m, &md, &y);
    return Py_BuildValue("(iii)", m, md, y);
}

static PyObject *
calendar2_absolute_from_kyureki(PyObject *self, PyObject *args)
{
    int m, l, md, y, j, a;
    QDATE q, q2;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(iiii)", &m, &l, &md, &y))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iiii", &m, &l, &md, &y))
	    return NULL;
    }
    CHECK_DOMAIN(y, 1);
    CHECK_DOMAIN2(m, 1, 12);
    CHECK_DOMAIN2(md, 1, 30);
    q.j = 0;
    q.y = y;
    q.yd = 0;
    q.m = m;
    q.md = md;
    q.wd = 0;
    q.leap = l;
    j = rqref(&q);
    a = absolute_from_julian_day_number(j);
    CHECK_RANGE(a, 1); /* 162193 - 767009 */
    qref(j, &q2);
    if (q.y != q2.y || q.m != q2.m || q.md != q2.md || q.leap != q2.leap) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("i", a);
}

static PyObject *
calendar2_kyureki_from_absolute(PyObject *self, PyObject *args)
{
    int a, j;
    QDATE q;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN2(a, 162193, 767009);
    j = julian_day_number_from_absolute(a);
    qref(j, &q);
    return Py_BuildValue("(iiii)", q.m, q.leap, q.md, q.y);
}

static PyObject *
calendar2_kyureki_day_of_week_from_absolute(PyObject *self, PyObject *args)
{
    int a, j;
    QDATE q;

    if (!PyArg_ParseTuple(args, "i", &a))
	return NULL;
    CHECK_DOMAIN2(a, 162193, 767009);
    j = julian_day_number_from_absolute(a);
    qref(j, &q);
    return Py_BuildValue("i", q.wd);
}

static PyMethodDef Calendar2Methods[] = {
    { "gregorian_leap_year", calendar2_gregorian_leap_year, 1 },
    { "gregorian_last_day_of_month", calendar2_gregorian_last_day_of_month, 1 },
    { "absolute_from_gregorian", calendar2_absolute_from_gregorian, 1 },
    { "gregorian_from_absolute", calendar2_gregorian_from_absolute, 1 },
    { "Kday_on_or_before", calendar2_Kday_on_or_before, 1 },
    { "absolute_from_iso", calendar2_absolute_from_iso, 1 },
    { "iso_from_absolute", calendar2_iso_from_absolute, 1 },
    { "absolute_from_calendar_week", calendar2_absolute_from_iso, 1 },
    { "calendar_week_from_absolute", calendar2_iso_from_absolute, 1 },
    { "julian_leap_year", calendar2_julian_leap_year, 1 },
    { "julian_last_day_of_month", calendar2_julian_last_day_of_month, 1 },
    { "absolute_from_julian", calendar2_absolute_from_julian, 1 },
    { "julian_from_absolute", calendar2_julian_from_absolute, 1 },
    { "islamic_leap_year", calendar2_islamic_leap_year, 1 },
    { "islamic_last_day_of_month", calendar2_islamic_last_day_of_month, 1 },
    { "absolute_from_islamic", calendar2_absolute_from_islamic, 1 },
    { "islamic_from_absolute", calendar2_islamic_from_absolute, 1 },
    { "hebrew_leap_year", calendar2_hebrew_leap_year, 1 },
    { "last_month_of_hebrew_year", calendar2_last_month_of_hebrew_year, 1 },
    { "hebrew_last_day_of_month", calendar2_hebrew_last_day_of_month, 1 },
    { "hebrew_calendar_elapsed_days", calendar2_hebrew_calendar_elapsed_days, 1 },
    { "days_in_hebrew_year", calendar2_days_in_hebrew_year, 1 },
    { "long_heshvan", calendar2_long_heshvan, 1 },
    { "short_kislev", calendar2_short_kislev, 1 },
    { "absolute_from_hebrew", calendar2_absolute_from_hebrew, 1 },
    { "hebrew_from_absolute", calendar2_hebrew_from_absolute, 1 },
    { "Nth_Kday", calendar2_Nth_Kday, 1 },
    { "nicaean_rule_easter", calendar2_nicaean_rule_easter, 1 },
    { "easter", calendar2_easter, 1 },
    { "absolute_from_mayan_long_count", calendar2_absolute_from_mayan_long_count, 1 },
    { "mayan_long_count_from_absolute", calendar2_mayan_long_count_from_absolute, 1 },
    { "mayan_haab_from_absolute", calendar2_mayan_haab_from_absolute, 1 },
    { "mayan_haab_difference", calendar2_mayan_haab_difference, 1 },
    { "mayan_haab_on_or_before", calendar2_mayan_haab_on_or_before, 1 },
    { "mayan_tzolkin_from_absolute", calendar2_mayan_tzolkin_from_absolute, 1 },
    { "mayan_tzolkin_difference", calendar2_mayan_tzolkin_difference, 1 },
    { "mayan_tzolkin_on_or_before", calendar2_mayan_tzolkin_on_or_before, 1 },
    { "mayan_haab_tzolkin_on_or_before", calendar2_mayan_haab_tzolkin_on_or_before, 1 },
    { "french_last_day_of_month", calendar2_french_last_day_of_month, 1 },
    { "french_leap_year", calendar2_french_leap_year, 1 },
    { "absolute_from_french", calendar2_absolute_from_french, 1 },
    { "french_from_absolute", calendar2_french_from_absolute, 1 },
    { "solar_longitude", calendar2_solar_longitude, 1 },
    { "zodiac", calendar2_zodiac, 1 },
    { "old_hindu_solar_from_absolute", calendar2_old_hindu_solar_from_absolute, 1 },
    { "absolute_from_old_hindu_solar", calendar2_absolute_from_old_hindu_solar, 1 },
    { "lunar_longitude", calendar2_lunar_longitude, 1 },
    { "lunar_phase", calendar2_lunar_phase, 1 },
    { "new_moon", calendar2_new_moon, 1 },
    { "old_hindu_lunar_from_absolute", calendar2_old_hindu_lunar_from_absolute, 1 },
    { "old_hindu_lunar_precedes", calendar2_old_hindu_lunar_precedes, 1 },
    { "absolute_from_old_hindu_lunar", calendar2_absolute_from_old_hindu_lunar, 1 },
    { "world_leap_year", calendar2_world_leap_year, 1 },
    { "world_last_day_of_month", calendar2_world_last_day_of_month, 1 },
    { "absolute_from_world", calendar2_absolute_from_world, 1 },
    { "world_from_absolute", calendar2_world_from_absolute, 1 },
    { "world_day_of_week_from_absolute", calendar2_world_day_of_week_from_absolute, 1 },
    { "coptic_leap_year", calendar2_coptic_leap_year, 1 },
    { "coptic_last_day_of_month", calendar2_coptic_last_day_of_month, 1 },
    { "absolute_from_coptic", calendar2_absolute_from_coptic, 1 },
    { "coptic_from_absolute", calendar2_coptic_from_absolute, 1 },
    { "ethiopian_leap_year", calendar2_ethiopian_leap_year, 1 },
    { "ethiopian_last_day_of_month", calendar2_ethiopian_last_day_of_month, 1 },
    { "absolute_from_ethiopian", calendar2_absolute_from_ethiopian, 1 },
    { "ethiopian_from_absolute", calendar2_ethiopian_from_absolute, 1 },
    { "jalaali_leap_year", calendar2_jalaali_leap_year, 1 },
    { "jalaali_last_day_of_month", calendar2_jalaali_last_day_of_month, 1 },
    { "absolute_from_jalaali", calendar2_absolute_from_jalaali, 1 },
    { "jalaali_from_absolute", calendar2_jalaali_from_absolute, 1 },
    { "absolute_from_ordinal_calendar", calendar2_absolute_from_ordinal_calendar, 1 },
    { "ordinal_calendar_from_absolute", calendar2_ordinal_calendar_from_absolute, 1 },
    { "absolute_from_julian_day_number", calendar2_absolute_from_julian_day_number, 1 },
    { "julian_day_number_from_absolute", calendar2_julian_day_number_from_absolute, 1 },
    { "absolute_from_jdn", calendar2_absolute_from_julian_day_number, 1 },
    { "jdn_from_absolute", calendar2_julian_day_number_from_absolute, 1 },
    { "julian_day_number_from_mjd", calendar2_julian_day_number_from_mjd, 1 },
    { "mjd_from_julian_day_number", calendar2_mjd_from_julian_day_number, 1 },
    { "jdn_from_mjd", calendar2_julian_day_number_from_mjd, 1 },
    { "mjd_from_jdn", calendar2_mjd_from_julian_day_number, 1 },
    { "absolute_from_lilian_day_number", calendar2_absolute_from_lilian_day_number, 1 },
    { "lilian_day_number_from_absolute", calendar2_lilian_day_number_from_absolute, 1 },
    { "absolute_from_ldn", calendar2_absolute_from_lilian_day_number, 1 },
    { "ldn_from_absolute", calendar2_lilian_day_number_from_absolute, 1 },
    { "day_of_week_from_absolute", calendar2_day_of_week_from_absolute, 1 },
    { "absolute_from_kyureki", calendar2_absolute_from_kyureki, 1 },
    { "kyureki_from_absolute", calendar2_kyureki_from_absolute, 1 },
    { "kyureki_day_of_week_from_absolute", calendar2_kyureki_day_of_week_from_absolute, 1 },
    { NULL, NULL }
};

PyMODINIT_FUNC
initcalendar2(void)
{
    PyObject *mod, *dict, *current_civil_month_names;

    mod = Py_InitModule("calendar2", Calendar2Methods);

    if (mod == NULL)
      return;

    dict = PyModule_GetDict(mod);
    (void)PyDict_SetItemString
	(dict, "__version__",
	 Py_BuildValue
	 ("s", "$Revision: 1.12 $"));
    (void)PyDict_SetItemString
	(dict, "day_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s]",
	  "Sunday", "Monday", "Tuesday", "Wednesday",
	  "Thursday", "Friday", "Saturday"));
    (void)PyDict_SetItemString
	(dict, "month_names",
	 current_civil_month_names = Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s,s,s]",
	  "*", "January", "February", "March", "April", "May", "June",
	  "July", "August", "September", "October", "November", "December"));
    (void)PyDict_SetItemString
	(dict, "gregorian_month_names", current_civil_month_names);
    (void)PyDict_SetItemString
	(dict, "julian_month_names", current_civil_month_names);
#ifndef PAPER
    (void)PyDict_SetItemString
	(dict, "islamic_month_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s,s,s]",
	  "*", "Muharram", "Safar", "Rabi` I", "Rabi` II", "Jumada I",
	  "Jumada II", "Rajab", "Sha`ban", "Ramadan", "Shawwal",
	  "Du al-Qa`da", "Du al-Hijja"));
#else
    (void)PyDict_SetItemString
	(dict, "islamic_month_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s,s,s]",
	  "*", "Muharram", "Safar", "Rabi I", "Rabi II", "Jumada I",
	  "Jumada II", "Rajab", "Sha'ban", "Ramadan", "Shawwal",
	  "Dhu al-Qada", "Dhu al-Hijjah"));
#endif
    (void)PyDict_SetItemString
	(dict, "hebrew_month_names",
	 Py_BuildValue
	 ("[[s,s,s,s,s,s,s,s,s,s,s,s,s][s,s,s,s,s,s,s,s,s,s,s,s,s,s]]",
	  "*", "Nisan", "Iyyar", "Sivan", "Tammuz", "Av", "Elul", "Tishri",
	  "Heshvan", "Kislev", "Teveth", "Shevat", "Adar",
	  "*", "Nisan", "Iyyar", "Sivan", "Tammuz", "Av", "Elul", "Tishri",
	  "Heshvan", "Kislev", "Teveth", "Shevat", "Adar I", "Adar II"));
    (void)PyDict_SetItemString
	(dict, "mayan_haab_month_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s]",
	  "*", "Pop", "Uo", "Zip", "Zotz", "Tzec", "Xul", "Yaxkin", "Mol",
	  "Chen", "Yax", "Zac", "Ceh", "Mac", "Kankin", "Muan", "Pax",
	  "Kayab", "Cumku", "Uayeb"));
    (void)PyDict_SetItemString
	(dict, "mayan_tzolkin_month_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s]",
	  "*", "Imix", "Ik", "Akbal", "Kan", "Chicchan", "Cimi", "Manik",
	  "Lamat", "Muluc", "Oc", "Chuen", "Eb", "Ben", "Ix", "Men", "Cib",
	  "Caban", "Etznab", "Cauac", "Ahau"));
#ifndef CS_LATIN1
    (void)PyDict_SetItemString
	(dict, "french_month_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s,s,s,s]",
	  "*", "Vende'miaire", "Brumaire", "Frimaire", "Nivo^se",
	  "Pluvio^se", "Vento^se", "Germinal", "Flore'al",
	  "Prairial", "Messidor", "Thermidor", "Fructidor", "*"));
#else
    (void)PyDict_SetItemString
	(dict, "french_month_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s,s,s,s]",
	  "*", "Vend\351miaire", "Brumaire", "Frimaire", "Niv\364se",
	  "Pluvi\364se", "Vent\364se", "Germinal", "Flor\351al",
	  "Prairial", "Messidor", "Thermidor", "Fructidor", "*"));
#endif
#ifndef PAPER
    (void)PyDict_SetItemString
	(dict, "french_day_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s]",
	  "*", "Primidi", "Doudi", "Tridi", "Quartidi", "Quintidi",
	  "Sextidi", "Septidi", "Octidi", "Nonidi", "Decadi"));
#else
    (void)PyDict_SetItemString
	(dict, "french_day_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s]",
	  "*", "Primidi", "Doudi", "Tridi", "Quartidi", "Quintidi",
	  "Sextidi", "Septidi", "Oxtidi", "Nonidi", "Decadi"));
#endif
#ifndef CS_LATIN1
    (void)PyDict_SetItemString
	(dict, "french_sansculottides_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s]",
	  "*", "Vertu", "Genie", "Labour", "Raison", "Recompense",
	  "Revolution", "*", "*", "*", "*"));
#else
    (void)PyDict_SetItemString
	(dict, "french_sansculottides_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s]",
	  "*", "Vertu", "G\351nie", "Labour", "Raison", "R\351compense",
	  "R\351volution", "*", "*", "*", "*"));
#endif
#ifndef PAPER
    (void)PyDict_SetItemString
	(dict, "old_hindu_solar_month_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s,s,s]",
	  "*", "Mesha", "Vrishabha", "Mithuna", "Karka", "Simha", "Kanya",
	  "Tula", "Vris'chika", "Dhanu", "Makara", "Kumbha", "Mina"));
#else
    (void)PyDict_SetItemString
	(dict, "old_hindu_solar_month_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s,s,s]",
	  "*", "Mesha", "Vrshabha", "Mithuna", "Karka", "Simha", "Kanya",
	  "Tula", "Vris'chika", "Dhanus", "Makara", "Kumbha", "Mina"));
#endif
    (void)PyDict_SetItemString
	(dict, "old_hindu_lunar_month_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s,s,s]",
	  "*", "Chaitra", "Vaisa'akha", "Jyaishtha", "Ashadha",
	  "S'ravana", "Bhadrapada", "As'vina", "Karttika", "Margas'ira",
	  "Pausha", "Magha", "Phalguna"));
    (void)PyDict_SetItemString
	(dict, "world_day_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s]",
	  "Sunday", "Monday", "Tuesday", "Wednesday",
	  "Thursday", "Friday", "Saturday", "World", "Leap"));
    (void)PyDict_SetItemString
	(dict, "world_month_names", current_civil_month_names);
    (void)PyDict_SetItemString
	(dict, "coptic_month_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s,s,s,s]",
	  "*", "Tut", "Babah", "Hatur", "Kiyahk", "Tubah", "Amshir",
	  "Baramhat", "Baramundah", "Bashans", "Ba'unah", "Abib", "Misra",
	  "al-Nasi"));
    (void)PyDict_SetItemString
	(dict, "ethiopian_month_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s,s,s,s]",
	  "*", "Maskaram", "Teqemt", "Khedar", "Takhs'as'", "Ter",
	  "Yakatit", "Magabit", "Miyazya", "Genbot", "Sane", "Hamle",
	  "Nahase", "Paguemen"));
    (void)PyDict_SetItemString
	(dict, "jalaali_month_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s,s,s]",
	  "*", "Farvardin", "Ordibehest", "Xordad", "Tir", "Mordad",
	  "Sahrivar", "Mehr", "Aban", "Azar", "Dey", "Bahman", "Esfand"));
    (void)PyDict_SetItemString
	(dict, "kyureki_day_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s]",
	  "Senkachi", "Tomobiki", "Semmake", "Butsumetsu", "Taian", "Shakku"));
    (void)PyDict_SetItemString
	(dict, "kyureki_month_names",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s,s,s]",
	  "*", "Mutsuki", "Kisaragi", "Yayoi", "Uzuki", "Satsuki",
	  "Minazuki", "Fuzuki", "Hazuki", "Nagatsuki", "Kannazuki",
	  "Shimotsuki", "Shiwasu"));
}
